(ns com.oly.static-sites.do-blog.pages.chess-board
  (:require
   [com.oly.static-sites.do-blog.pages.chess.movement
    :refer [fetch-selected-piece-moves]]
   [com.oly.static-sites.do-blog.pages.chess.render
    :refer [calculate-black-white]]
   [com.oly.static-sites.do-blog.pages.chess.state
    :refer [black-square start-state white-square]]
   [com.oly.static-sites.ui-hiccup.interface :refer [board]]
   [reagent.core :as r]))

(defonce game-state (r/atom []))
(defonce game-state-viking (r/atom []))

(defn board-test [size]
  (loop [x-pos 1
         y-pos 1
         result []]
    (if (and (= x-pos size) (= y-pos size))
      result
      (if (= size x-pos)
        (recur 1
               (inc y-pos)
               (conj result [x-pos y-pos]))
        (recur (inc x-pos)
               y-pos
               (conj result [x-pos y-pos]))))))

(defn reset-square [pos value]
  (prn (str "update " pos))
  (swap! game-state assoc pos value))

(defn update-square [pos value]
  (prn (str "update " pos))
  (swap! game-state assoc pos
         (merge (get @game-state pos {})
                value)))

;; (update-square [1 1] {:class "fl bg-green"})
;; (update-square [2 2] {:class "fl bg-green"})

(defn square-defaults [pos]
  {:class (if (= (calculate-black-white pos) :black)
            "bg-black-20 black-80"
            "bg-black-80 white-80")})

(defn create-board [size]
  (->> (board-test size)
       (mapv #(let [square-colour (calculate-black-white %)]
                (vector % (square-defaults %)
                        #_{;:colour (calculate-black-white %)
                           :class (if (= square-colour :black)
                                    "bg-white-80 black-80"
                                    "bg-black-60 white-80")})))
       (into {})))

(defn highlight-moves [pos]
  (->> (fetch-selected-piece-moves (first (filter (fn [[k v]] (:selected v)) @game-state)))
       (mapv (fn [pos]
               (update-square
                pos
                {:highlight " bg-green "})))))

(defn clear-keys [key]
  (reset! game-state
          (reduce-kv (fn [m pos piece]
                       (assoc m pos (dissoc piece key))) {} @game-state)))

(defn get-selected-square []
  (last (filter (fn [[_ square]] (true? (:selected square))) @game-state)))

(defn valid-move? [dest-pos]
  (let [piece (first (filter (fn [[k v]] (:selected v)) @game-state))
        moves (fetch-selected-piece-moves piece)]
    (prn (some (fn [p] (= p dest-pos)) moves))
    (some (fn [p] (= p dest-pos)) moves)))

(defn move-piece [src-pos piece dest-pos]
  (prn (str "move piece from " src-pos " to " dest-pos))
  (prn piece)
  (when (valid-move? dest-pos)
    (update-square dest-pos piece)
    (reset-square src-pos (square-defaults src-pos))))

(defn click-square [evt p]
  (let [[selected-pos selected-piece] (get-selected-square)]
    (clear-keys :highlight)

    (if selected-pos
      (do (prn "previously selected piece")
          (move-piece (:pos selected-piece) selected-piece (:pos p))
          (clear-keys :selected))
      (do
        (prn "no previously selected piece")
        (clear-keys :selected)
        (update-square (:pos p) (assoc p :selected true))
        (highlight-moves (:pos p)))))

  (clear-keys :selected)
  (clear-keys :highlight)
  (update-square (:pos p) (assoc p :selected true))
  (highlight-moves (:pos p))
  (prn (get @game-state (:pos p))))

(defn enhance-board [board]
  (reduce-kv
   (fn [m key square]
     (assoc m key (assoc square :on-click click-square :pos key)))
   {}
   board))

(defn place-pieces-on-board [board pieces]
  (reduce (fn [m piece]
            (let [square-colour (calculate-black-white (:start-pos piece))]
              (assoc m (:start-pos piece)
                     (-> piece
                         (assoc :pos (:start-pos piece)
                                :text (:character piece)
                                :selected false
                                ;; :colour (if (= square-colour :black)
                                ;;           white-square
                                ;;           black-square)
                                :on-click click-square
                                :class (if (= square-colour :black)
                                         (str "bg-black-20 " (if (= :white (:colour piece))
                                                               " black "
                                                               " white "))

                                         (str "bg-black-80 " (if (= :white (:colour piece)) " black" " white "))))))))

          board
          pieces))

(reset! game-state-viking
        (-> (create-board 11)))

(reset! game-state
        (-> (create-board 8)
            (enhance-board)
            (place-pieces-on-board start-state)))

(defn chess-page [{:keys [path-params] :as route}]
  (fn [{:keys [path-params] :as route}]
    [:div
     (let [[pos piece] (get-selected-square)]
       [:div
        [:div (str "pos = " pos)]
        [:div (str (:type piece))]
        [:div (str piece)]
        [:div (str "move = " (fetch-selected-piece-moves [pos piece]))]])
     [:div (str (fetch-selected-piece-moves  (first (filter (fn [[k v]] (:selected v)) @game-state))))]
     [:div (board 8 @game-state)]
     [:div "Viking chess board"]
     [:div (board 11 @game-state-viking)]]))

(comment
  (get-selected-square)
  @game-state)
